/******************************************************************

This program is made to verify Table 2 of paper, regime testing.

Jin Seo Cho

Dec. 05, 2004

*******************************************************************/

new;
library pgraph cml;
#include cml.ext;
CMLset;
_pdate = {0};

a = time;
proc lpr2(x, dta);
    local p1, the1, the2, the0, ss, nn, dep, ind;
    nn   = rows(dta);
    dep  = dta[2:nn,1];
    ind  = dta[1:nn-1,1];
    the1 = x[1];
    the2 = x[2];
    the0 = x[3]; 
    ss   = x[4];
    p1   = x[5];

    retp( ln( p1 .* exp( lnpdfmvn(dep-the1-the0*ind,ss) )
        + (1-p1) .* exp( lnpdfmvn(dep-the2-the0*ind,ss) ) ) );

endp;

__title = "Estimation of mxiture model with 2 components";
_cml_DirTol = 0.001;
_cml_Bounds = { -2 2,
                -2 2,
                -1 1.,
                .0 5.,
                .0 1. };

_cml_GradProc = 0;
__output = 0;

kkk = 3000;
nn  = 500;
nnn = nn;

/********************************************************************************************************************/

/* Parameters */

p1  = 0.9;
q1  = 1-p1;

trv = 0.2;
rho = 0.5;
pp  = (1-q1)/(2-p1-q1);

tes1 = zeros(kkk,3);
tes2 = zeros(kkk,3);

j = 1;
do until j > kkk;

    /* HMM of AR(1) */

    sss = zeros(nn,1);
    rrr = zeros(nn,1);
    xxx = zeros(nn,1);

    sss = rndu(nn,1);
    if sss[1,1] < pp; rrr[1,1] = -trv; else; rrr[1,1] = trv; endif;
    iii = 2;
    do until iii > nn;
        if rrr[iii-1,1] == trv;
            if sss[iii,1] < p1;
                rrr[iii,1] = -trv;
            else;
                rrr[iii,1] = trv;
            endif;
        else;
            if sss[iii,1] < q1;
                rrr[iii,1] = trv;
            else;
                rrr[iii,1] = -trv;
            endif;
        endif;             
        iii = iii + 1;
    endo;

    aaa = rndn(nn,1);
    qa  = rndu(1,1);
    if qa < pp;
        xxx[1,1] = -trv + aaa[1,1]/sqrt(1-rho^2);
    else;
        xxx[1,1] = trv + aaa[1,1]/sqrt(1-rho^2);
    endif;
    iii = 2;
    do until iii > nn;
        xxx[iii,1] = rrr[iii,1] + rho * xxx[iii-1,1] + aaa[iii,1];
        iii = iii + 1;
    endo;

    /* Estimation */

    /* null model estimation */

    indv  = ones(nnn-1,1)~xxx[1:nnn-1,1];
    depv  = xxx[2:nnn,1];
    bhatn = inv(indv'*indv)*indv'*depv;
    uhatn = depv - indv*bhatn;
    shatn = vcx(uhatn);

    /* Initial parameter given by guess */

    /* alternative model estimation */

    x = zeros(1,5);
    x[1] = -trv;
    x[2] = trv;
    x[3] = rho;
    x[4] = 1.0;
    x[5] = pp;

    {b,h,g,cov,ret}=CML(xxx,0,&lpr2,x);

    bhat1 = b[1];
    bhat2 = b[2];
    bhat0 = b[3];
    shata = b[4];
    phat  = b[5];

    bha1a = zeros(2,1); bha1a[1,1] = bhat1; bha1a[2,1] = bhat0;
    bha2a = zeros(2,1); bha2a[1,1] = bhat2; bha2a[2,1] = bhat0;

    lhooda = ln(      phat * exp(-0.5 * (depv - indv * bha1a)^2/shata )/sqrt(2*pi*shata) 
                + (1-phat) * exp(-0.5 * (depv - indv * bha2a)^2/shata )/sqrt(2*pi*shata)  );
    lhoodn = ln(  exp(-0.5 * (depv - indv*bhatn)^2/shatn )/sqrt(2*pi*shatn) );

    tes2[j,1] = j/kkk;
    tes2[j,2] = 2 * ( sumc( lhooda ) - sumc( lhoodn ) );
    j = j + 1;
endo;
    
te5p0 = (tes2[.,1] .> 5.6142);

pow11 = meanc(te5p0);
print pow11;
col1 = tes2[.,1:2];

/********************************************************************************************************************/

/* Parameters */

trv = 0.4;
rho = 0.5;
pp  = (1-q1)/(2-p1-q1);

tes1 = zeros(kkk,3);
tes2 = zeros(kkk,3);

j = 1;
do until j > kkk;

    /* HMM of AR(1) */

    sss = zeros(nn,1);
    rrr = zeros(nn,1);
    xxx = zeros(nn,1);

    sss = rndu(nn,1);
    if sss[1,1] < pp; rrr[1,1] = -trv; else; rrr[1,1] = trv; endif;
    iii = 2;
    do until iii > nn;
        if rrr[iii-1,1] == trv;
            if sss[iii,1] < p1;
                rrr[iii,1] = -trv;
            else;
                rrr[iii,1] = trv;
            endif;
        else;
            if sss[iii,1] < q1;
                rrr[iii,1] = trv;
            else;
                rrr[iii,1] = -trv;
            endif;
        endif;             
        iii = iii + 1;
    endo;

    aaa = rndn(nn,1);
    qa  = rndu(1,1);
    if qa < pp;
        xxx[1,1] = -trv + aaa[1,1]/sqrt(1-rho^2);
    else;
        xxx[1,1] = trv + aaa[1,1]/sqrt(1-rho^2);
    endif;
    iii = 2;
    do until iii > nn;
        xxx[iii,1] = rrr[iii,1] + rho * xxx[iii-1,1] + aaa[iii,1];
        iii = iii + 1;
    endo;

    /* Estimation */

    /* null model estimation */

    indv  = ones(nnn-1,1)~xxx[1:nnn-1,1];
    depv  = xxx[2:nnn,1];
    bhatn = inv(indv'*indv)*indv'*depv;
    uhatn = depv - indv*bhatn;
    shatn = vcx(uhatn);

    /* Initial parameter given by guess */

    /* alternative model estimation */

    x = zeros(1,5);
    x[1] = -trv;
    x[2] = trv;
    x[3] = rho;
    x[4] = 1.0;
    x[5] = pp;

    {b,h,g,cov,ret}=CML(xxx,0,&lpr2,x);

    bhat1 = b[1];
    bhat2 = b[2];
    bhat0 = b[3];
    shata = b[4];
    phat  = b[5];

    bha1a = zeros(2,1); bha1a[1,1] = bhat1; bha1a[2,1] = bhat0;
    bha2a = zeros(2,1); bha2a[1,1] = bhat2; bha2a[2,1] = bhat0;

    lhooda = ln(      phat * exp(-0.5 * (depv - indv * bha1a)^2/shata )/sqrt(2*pi*shata) 
                + (1-phat) * exp(-0.5 * (depv - indv * bha2a)^2/shata )/sqrt(2*pi*shata)  );
    lhoodn = ln(  exp(-0.5 * (depv - indv*bhatn)^2/shatn )/sqrt(2*pi*shatn) );

    tes2[j,1] = 2 * ( sumc( lhooda ) - sumc( lhoodn ) );

    j = j + 1;
endo;
    
te5p0 = (tes2[.,1] .> 5.6142);

pow12 = meanc(te5p0);
print pow12;
col2 = tes2[.,1];

/********************************************************************************************************************/

/* Parameters */

trv = 0.6;
rho = 0.5;
pp  = (1-q1)/(2-p1-q1);

tes1 = zeros(kkk,3);
tes2 = zeros(kkk,3);

j = 1;
do until j > kkk;

    /* HMM of AR(1) */

    sss = zeros(nn,1);
    rrr = zeros(nn,1);
    xxx = zeros(nn,1);

    sss = rndu(nn,1);
    if sss[1,1] < pp; rrr[1,1] = -trv; else; rrr[1,1] = trv; endif;
    iii = 2;
    do until iii > nn;
        if rrr[iii-1,1] == trv;
            if sss[iii,1] < p1;
                rrr[iii,1] = -trv;
            else;
                rrr[iii,1] = trv;
            endif;
        else;
            if sss[iii,1] < q1;
                rrr[iii,1] = trv;
            else;
                rrr[iii,1] = -trv;
            endif;
        endif;             
        iii = iii + 1;
    endo;

    aaa = rndn(nn,1);
    qa  = rndu(1,1);
    if qa < pp;
        xxx[1,1] = -trv + aaa[1,1]/sqrt(1-rho^2);
    else;
        xxx[1,1] = trv + aaa[1,1]/sqrt(1-rho^2);
    endif;
    iii = 2;
    do until iii > nn;
        xxx[iii,1] = rrr[iii,1] + rho * xxx[iii-1,1] + aaa[iii,1];
        iii = iii + 1;
    endo;

    /* Estimation */

    /* null model estimation */

    indv  = ones(nnn-1,1)~xxx[1:nnn-1,1];
    depv  = xxx[2:nnn,1];
    bhatn = inv(indv'*indv)*indv'*depv;
    uhatn = depv - indv*bhatn;
    shatn = vcx(uhatn);

    /* Initial parameter given by guess */

    /* alternative model estimation */

    x = zeros(1,5);
    x[1] = -trv;
    x[2] = trv;
    x[3] = rho;
    x[4] = 1.0;
    x[5] = pp;

    {b,h,g,cov,ret}=CML(xxx,0,&lpr2,x);

    bhat1 = b[1];
    bhat2 = b[2];
    bhat0 = b[3];
    shata = b[4];
    phat  = b[5];

    bha1a = zeros(2,1); bha1a[1,1] = bhat1; bha1a[2,1] = bhat0;
    bha2a = zeros(2,1); bha2a[1,1] = bhat2; bha2a[2,1] = bhat0;

    lhooda = ln(      phat * exp(-0.5 * (depv - indv * bha1a)^2/shata )/sqrt(2*pi*shata) 
                + (1-phat) * exp(-0.5 * (depv - indv * bha2a)^2/shata )/sqrt(2*pi*shata)  );
    lhoodn = ln(  exp(-0.5 * (depv - indv*bhatn)^2/shatn )/sqrt(2*pi*shatn) );

    tes2[j,1] = 2 * ( sumc( lhooda ) - sumc( lhoodn ) );

    j = j + 1;
endo;
    
te5p0 = (tes2[.,1] .> 5.6142);

pow13 = meanc(te5p0);
print pow13;
col3 = tes2[.,1];
/********************************************************************************************************************/

/* Parameters */

trv = 0.8;
rho = 0.5;
pp  = (1-q1)/(2-p1-q1);

tes1 = zeros(kkk,3);
tes2 = zeros(kkk,3);

j = 1;
do until j > kkk;

    /* HMM of AR(1) */

    sss = zeros(nn,1);
    rrr = zeros(nn,1);
    xxx = zeros(nn,1);

    sss = rndu(nn,1);
    if sss[1,1] < pp; rrr[1,1] = -trv; else; rrr[1,1] = trv; endif;
    iii = 2;
    do until iii > nn;
        if rrr[iii-1,1] == trv;
            if sss[iii,1] < p1;
                rrr[iii,1] = -trv;
            else;
                rrr[iii,1] = trv;
            endif;
        else;
            if sss[iii,1] < q1;
                rrr[iii,1] = trv;
            else;
                rrr[iii,1] = -trv;
            endif;
        endif;             
        iii = iii + 1;
    endo;

    aaa = rndn(nn,1);
    qa  = rndu(1,1);
    if qa < pp;
        xxx[1,1] = -trv + aaa[1,1]/sqrt(1-rho^2);
    else;
        xxx[1,1] = trv + aaa[1,1]/sqrt(1-rho^2);
    endif;
    iii = 2;
    do until iii > nn;
        xxx[iii,1] = rrr[iii,1] + rho * xxx[iii-1,1] + aaa[iii,1];
        iii = iii + 1;
    endo;

    /* Estimation */

    /* null model estimation */

    indv  = ones(nnn-1,1)~xxx[1:nnn-1,1];
    depv  = xxx[2:nnn,1];
    bhatn = inv(indv'*indv)*indv'*depv;
    uhatn = depv - indv*bhatn;
    shatn = vcx(uhatn);

    /* Initial parameter given by guess */

    /* alternative model estimation */

    x = zeros(1,5);
    x[1] = -trv;
    x[2] = trv;
    x[3] = rho;
    x[4] = 1.0;
    x[5] = pp;

    {b,h,g,cov,ret}=CML(xxx,0,&lpr2,x);

    bhat1 = b[1];
    bhat2 = b[2];
    bhat0 = b[3];
    shata = b[4];
    phat  = b[5];

    bha1a = zeros(2,1); bha1a[1,1] = bhat1; bha1a[2,1] = bhat0;
    bha2a = zeros(2,1); bha2a[1,1] = bhat2; bha2a[2,1] = bhat0;

    lhooda = ln(      phat * exp(-0.5 * (depv - indv * bha1a)^2/shata )/sqrt(2*pi*shata) 
                + (1-phat) * exp(-0.5 * (depv - indv * bha2a)^2/shata )/sqrt(2*pi*shata)  );
    lhoodn = ln(  exp(-0.5 * (depv - indv*bhatn)^2/shatn )/sqrt(2*pi*shatn) );

    tes2[j,1] = 2 * ( sumc( lhooda ) - sumc( lhoodn ) );

    j = j + 1;
endo;
    
te5p0 = (tes2[.,1] .> 5.6142);

pow14 = meanc(te5p0);
print pow14;
col4 = tes2[.,1];

/********************************************************************************************************************/

/* Parameters */

trv = 1.;
rho = 0.5;
pp  = (1-q1)/(2-p1-q1);

tes1 = zeros(kkk,3);
tes2 = zeros(kkk,3);

j = 1;
do until j > kkk;

    /* HMM of AR(1) */

    sss = zeros(nn,1);
    rrr = zeros(nn,1);
    xxx = zeros(nn,1);

    sss = rndu(nn,1);
    if sss[1,1] < pp; rrr[1,1] = -trv; else; rrr[1,1] = trv; endif;
    iii = 2;
    do until iii > nn;
        if rrr[iii-1,1] == trv;
            if sss[iii,1] < p1;
                rrr[iii,1] = -trv;
            else;
                rrr[iii,1] = trv;
            endif;
        else;
            if sss[iii,1] < q1;
                rrr[iii,1] = trv;
            else;
                rrr[iii,1] = -trv;
            endif;
        endif;             
        iii = iii + 1;
    endo;

    aaa = rndn(nn,1);
    qa  = rndu(1,1);
    if qa < pp;
        xxx[1,1] = -trv + aaa[1,1]/sqrt(1-rho^2);
    else;
        xxx[1,1] = trv + aaa[1,1]/sqrt(1-rho^2);
    endif;
    iii = 2;
    do until iii > nn;
        xxx[iii,1] = rrr[iii,1] + rho * xxx[iii-1,1] + aaa[iii,1];
        iii = iii + 1;
    endo;

    /* Estimation */

    /* null model estimation */

    indv  = ones(nnn-1,1)~xxx[1:nnn-1,1];
    depv  = xxx[2:nnn,1];
    bhatn = inv(indv'*indv)*indv'*depv;
    uhatn = depv - indv*bhatn;
    shatn = vcx(uhatn);

    /* Initial parameter given by guess */

    /* alternative model estimation */

    x = zeros(1,5);
    x[1] = -trv;
    x[2] = trv;
    x[3] = rho;
    x[4] = 1.0;
    x[5] = pp;

    {b,h,g,cov,ret}=CML(xxx,0,&lpr2,x);

    bhat1 = b[1];
    bhat2 = b[2];
    bhat0 = b[3];
    shata = b[4];
    phat  = b[5];

    bha1a = zeros(2,1); bha1a[1,1] = bhat1; bha1a[2,1] = bhat0;
    bha2a = zeros(2,1); bha2a[1,1] = bhat2; bha2a[2,1] = bhat0;

    lhooda = ln(      phat * exp(-0.5 * (depv - indv * bha1a)^2/shata )/sqrt(2*pi*shata) 
                + (1-phat) * exp(-0.5 * (depv - indv * bha2a)^2/shata )/sqrt(2*pi*shata)  );
    lhoodn = ln(  exp(-0.5 * (depv - indv*bhatn)^2/shatn )/sqrt(2*pi*shatn) );

    tes2[j,1] = 2 * ( sumc( lhooda ) - sumc( lhoodn ) );

    j = j + 1;
endo;
    
te5p0 = (tes2[.,1] .> 5.6142);

pow15 = meanc(te5p0);
print pow15;
col5 = tes2[.,1];

col1[.,2] = sortc(col1[.,2],1);
col2[.,1] = sortc(col2[.,1],1);
col3[.,1] = sortc(col3[.,1],1);
col4[.,1] = sortc(col4[.,1],1);
col5[.,1] = sortc(col5[.,1],1);

output file = table4_5th_pn.txt reset;                          /*************** File Name *******************/
screen off;
print col1~col2~col3~col4~col5;
screen on;
output off;
